@@ -6,15 +6,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
- * program is free software; you may redistribute and/or modify it under   *
- * the terms of the GNU General Public License as published by the Free    *
+ * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
+ * is also a registered trademark of Insecure.Com LLC.  This program is    *
+ * free software; you may redistribute and/or modify it under the          *
+ * terms of the GNU General Public License as published by the Free        *
  * Software Foundation; Version 2.  This guarantees your right to use,     *
  * modify, and redistribute this software under certain conditions.  If    *
  * you wish to embed Nmap technology into proprietary software, we may be  *
  * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as   *
- * our remote OS fingerprinting database and code.                         *
+ * Many security scanner vendors already license Nmap technology such as  *
+ * our remote OS fingerprinting database and code, service/version         *
+ * detection system, and port scanning code.                               *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -83,7 +85,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap.cc,v 1.37 2003/09/20 09:03:00 fyodor Exp $ */
+/* $Id: nmap.cc,v 1.45 2004/07/07 06:08:09 fyodor Exp $ */
 
 #include "nmap.h"
 #include "osscan.h"
@@ -239,6 +241,7 @@ int nmap_main(int argc, char *argv[]) {
     {"iL", required_argument, 0, 0},  
     {"iR", required_argument, 0, 0},
     {"sI", required_argument, 0, 0},  
+    {"source_port", required_argument, 0, 'g'},
     {"randomize_hosts", no_argument, 0, 0},
     {"osscan_limit", no_argument, 0, 0}, /* skip OSScan if no open ports */
     {"osscan_guess", no_argument, 0, 0}, /* More guessing flexability */
@@ -405,13 +408,11 @@ int nmap_main(int argc, char *argv[]) {
 	}
 	if (!strcmp(optarg, "-")) {
 	  inputfd = stdin;
-	  log_write(LOG_STDOUT, "Reading target specifications from stdin\n");
 	} else {    
 	  inputfd = fopen(optarg, "r");
 	  if (!inputfd) {
 	    fatal("Failed to open input file %s for reading", optarg);
 	  }  
-	  log_write(LOG_STDOUT, "Reading target specifications from FILE: %s\n", optarg);
 	}
       } else if (strcmp(long_options[option_index].name, "iR") == 0) {
 	o.generate_random_ips = 1;
@@ -496,13 +497,11 @@ int nmap_main(int argc, char *argv[]) {
       }
       if (!strcmp(optarg, "-")) {
 	inputfd = stdin;
-	log_write(LOG_STDOUT, "Reading target specifications from stdin\n");
       } else {    
 	inputfd = fopen(optarg, "r");
 	if (!inputfd) {
 	  fatal("Failed to open input file %s for reading", optarg);
 	}  
-	log_write(LOG_STDOUT, "Reading target specifications from FILE: %s\n", optarg);
       }
       break;  
     case 'M': 
@@ -515,7 +514,6 @@ int nmap_main(int argc, char *argv[]) {
     case 'm': 
       machinefilename = optarg;
       break;
-    case 'N': o.force++; break;
     case 'n': o.noresolve++; break;
     case 'O': 
       o.osscan++; 
@@ -584,8 +582,9 @@ int nmap_main(int argc, char *argv[]) {
 	  o.num_ping_ackprobes = 1;
 	  o.ping_ackprobes[0] = DEFAULT_TCP_PROBE_PORT;
 	}
-      }
-      else { 
+      } else if (*optarg == 'O') {
+	fatal("-PO (the letter O)? No such option. Perhaps you meant to disable pings with -P0 (Zero).");
+      } else { 
 	fatal("Illegal Argument to -P, use -P0, -PI, -PB, -PM, -PP, -PT, or -PT80 (or whatever number you want for the TCP probe destination port)"); 
       }
       break;
@@ -1006,6 +1005,7 @@ int nmap_main(int argc, char *argv[]) {
 		    currenths->targetipstr(), currenths->HostName());
 	} else {
 	  printportoutput(currenths, &currenths->ports);
+	  printmacinfo(currenths);
 	  printosscanoutput(currenths);
  	}      
 
@@ -1234,9 +1234,9 @@ struct scan_lists *getpts(char *origexpr) {
       if (rangestart < 0 || rangestart > 65535) {
 	fatal("Ports to be scanned must be between 0 and 65535 inclusive");
       }
-      if (rangestart == 0) {
+/*      if (rangestart == 0) {
 	error("WARNING:  Scanning \"port 0\" is supported, but unusual.");
-      }
+      } */
       current_range = endptr;
       while(isspace((int) *current_range)) current_range++;
     } else {
@@ -1344,7 +1344,7 @@ void printusage(char *name, int rc) {
 	 "  -sR/-I RPC/Identd scan (use with other scan types)\n"
 	 "Some Common Options (none are required, most can be combined):\n"
 	 "* -O Use TCP/IP fingerprinting to guess remote operating system\n"
-	 "  -p <range> ports to scan.  Example range: '1-1024,1080,6666,31337'\n"
+	 "  -p <range> ports to scan.  Example range: 1-1024,1080,6666,31337\n"
 	 "  -F Only scans ports listed in nmap-services\n"
 	 "  -v Verbose. Its use is recommended.  Use twice for greater effect.\n"
 	 "  -P0 Don't ping hosts (needed to scan www.microsoft.com and others)\n"
@@ -1713,6 +1713,7 @@ int getidentinfoz(struct in_addr target, u16 localport, u16 remoteport,
   char response[1024];
   char *p,*q;
   char  *os;
+  int slen = 0;
 
   if (ownersz == 0) return 0;
   owner[0] = '\0';
@@ -1763,6 +1764,8 @@ int getidentinfoz(struct in_addr target, u16 localport, u16 remoteport,
 	    if ((q = strchr(p, '\r'))) *q = '\0';
 	    if ((q = strchr(p, '\n'))) *q = '\0';
 	    Strncpy(owner, p, ownersz);
+            slen = strlen(owner);
+	    replacenonprintable(owner, slen, '.');
 	  }
 	}
       } 
@@ -1830,7 +1833,7 @@ int ftp_anon_connect(struct ftpinfo *ftp) {
     exit(1);
   }
   if (o.verbose || o.debugging) log_write(LOG_STDOUT, "Connected:");
-  while ((res = recvtime(sd, recvbuf, sizeof(recvbuf) - 1,7)) > 0) 
+  while ((res = recvtime(sd, recvbuf, sizeof(recvbuf) - 1,7, NULL)) > 0) 
     if (o.debugging || o.verbose) {
       recvbuf[res] = '\0';
       log_write(LOG_STDOUT, "%s", recvbuf);
@@ -1843,7 +1846,7 @@ int ftp_anon_connect(struct ftpinfo *ftp) {
   snprintf(command, 511, "USER %s\r\n", ftp->user);
 
   send(sd, command, strlen(command), 0);
-  res = recvtime(sd, recvbuf, sizeof(recvbuf) - 1,12);
+  res = recvtime(sd, recvbuf, sizeof(recvbuf) - 1,12, NULL);
   if (res <= 0) {
     perror("recv problem from ftp bounce server");
     exit(1);
@@ -1859,7 +1862,7 @@ int ftp_anon_connect(struct ftpinfo *ftp) {
   snprintf(command, 511, "PASS %s\r\n", ftp->pass);
 
   send(sd, command, strlen(command), 0);
-  res = recvtime(sd, recvbuf, sizeof(recvbuf) - 1,12);
+  res = recvtime(sd, recvbuf, sizeof(recvbuf) - 1,12, NULL);
   if (res < 0) {
     perror("recv problem from ftp bounce server\n");
     exit(1);
@@ -1874,7 +1877,7 @@ int ftp_anon_connect(struct ftpinfo *ftp) {
       exit(1);
     }
   }
-  while ((res = recvtime(sd, recvbuf, sizeof(recvbuf) - 1,2)) > 0) 
+  while ((res = recvtime(sd, recvbuf, sizeof(recvbuf) - 1,2, NULL)) > 0) 
     if (o.debugging) {
       recvbuf[res] = '\0';
       log_write(LOG_STDOUT, "%s", recvbuf);
