@@ -9,15 +9,17 @@
  *                                                                         *
  ***********************IMPORTANT NMAP LICENSE TERMS************************
  *                                                                         *
- * The Nmap Security Scanner is (C) 1995-2003 Insecure.Com LLC. This       *
- * program is free software; you may redistribute and/or modify it under   *
- * the terms of the GNU General Public License as published by the Free    *
+ * The Nmap Security Scanner is (C) 1996-2004 Insecure.Com LLC. Nmap       *
+ * is also a registered trademark of Insecure.Com LLC.  This program is    *
+ * free software; you may redistribute and/or modify it under the          *
+ * terms of the GNU General Public License as published by the Free        *
  * Software Foundation; Version 2.  This guarantees your right to use,     *
  * modify, and redistribute this software under certain conditions.  If    *
  * you wish to embed Nmap technology into proprietary software, we may be  *
  * willing to sell alternative licenses (contact sales@insecure.com).      *
- * Many security scanner vendors already license Nmap technology such as   *
- * our remote OS fingerprinting database and code.                         *
+ * Many security scanner vendors already license Nmap technology such as  *
+ * our remote OS fingerprinting database and code, service/version         *
+ * detection system, and port scanning code.                               *
  *                                                                         *
  * Note that the GPL places important restrictions on "derived works", yet *
  * it does not provide a detailed definition of that term.  To avoid       *
@@ -86,12 +88,13 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: output.cc,v 1.26 2003/09/20 09:03:00 fyodor Exp $ */
+/* $Id: output.cc,v 1.31 2004/04/19 02:02:23 fyodor Exp $ */
 
 #include "output.h"
 #include "osscan.h"
 #include "NmapOps.h"
 #include "NmapOutputTable.h"
+#include "MACLookup.h"
 
 #include <string>
 
@@ -203,6 +206,9 @@ void printportoutput(Target *currenths, PortList *plist) {
   char rpcmachineinfo[64];
   char portinfo[64];
   char xmlbuf[512];
+  char grepvers[256];
+  char grepown[64];
+  char *p;
   char *state;
   char serviceinfo[64];
   char *name=NULL;
@@ -379,9 +385,39 @@ void printportoutput(Target *currenths, PortList *plist) {
 	if (*sd.fullversion)
 	  Tbl->addItem(rowno, versioncol, true, sd.fullversion);
 
-	log_write(LOG_MACHINE,"%d/%s/%s/%s/%s/%s//", current->portno, state, 
-		  protocol, (current->owner)? current->owner : "",
-		  (sd.name)? sd.name: "", rpcmachineinfo);    
+	// How should we escape illegal chars in grepable output?
+	// Well, a reasonably clean way would be backslash escapes
+	// such as \/ and \\ .  // But that makes it harder to pick
+	// out fields with awk, cut, and such.  So I'm gonna use the
+	// ugly hat (fitting to grepable output) or replacing the '/'
+	// character with '|' in the version and owner fields.
+	Strncpy(grepvers, sd.fullversion, 
+		sizeof(grepvers) / sizeof(*grepvers));
+	p = grepvers;
+	while((p = strchr(p, '/'))) {
+	  *p = '|';
+	  p++;
+	}
+	if (!current->owner) *grepown = '\0';
+	else {
+	  Strncpy(grepown, current->owner, 
+		  sizeof(grepown) / sizeof(*grepown));
+	  p = grepown;
+	  while((p = strchr(p, '/'))) {
+	    *p = '|';
+	    p++;
+	  }
+	}
+	if (!sd.name) serviceinfo[0] = '\0';
+	else {
+	  p = serviceinfo;
+	  while((p = strchr(p, '/'))) {
+	    *p = '|';
+	    p++;
+	  }
+	}
+	log_write(LOG_MACHINE,"%d/%s/%s/%s/%s/%s/%s/", current->portno, state, 
+		  protocol, grepown, serviceinfo, rpcmachineinfo, grepvers);    
 	
 	log_write(LOG_XML, "<port protocol=\"%s\" portid=\"%d\">", protocol, current->portno);
 	log_write(LOG_XML, "<state state=\"%s\" />", state);
@@ -817,7 +853,7 @@ static void printosclassificationoutput(const struct OS_Classification_Results *
       if (OSR->OSC[classno]->OS_Generation) {
 	snprintf(tmpbuf, sizeof(tmpbuf), " osgen=\"%s\"", OSR->OSC[classno]->OS_Generation);
       } else tmpbuf[0] = '\0';
-      log_write(LOG_XML, "<osclass type=\"%s\" vendor=\"%s\" osfamily=\"%s\"%s accuracy=\"%d\"/>\n", OSR->OSC[classno]->Device_Type, OSR->OSC[classno]->OS_Vendor, OSR->OSC[classno]->OS_Family, tmpbuf, (int) (OSR->OSC_Accuracy[classno] * 100));
+      log_write(LOG_XML, "<osclass type=\"%s\" vendor=\"%s\" osfamily=\"%s\"%s accuracy=\"%d\" />\n", OSR->OSC[classno]->Device_Type, OSR->OSC[classno]->OS_Vendor, OSR->OSC[classno]->OS_Family, tmpbuf, (int) (OSR->OSC_Accuracy[classno] * 100));
     }
 
     // Now to create the fodder for normal output
@@ -883,6 +919,23 @@ static void printosclassificationoutput(const struct OS_Classification_Results *
   return;
 }
 
+/* Prints the MAC address if one was found for the target (generally
+   this means that the target is directly connected on an ethernet
+   network. */
+void printmacinfo(Target *currenths) {
+  const u8 *mac = currenths->MACAddress();
+  char macascii[32];
+  char vendorstr[128];
+
+  if (mac) {
+    const char *macvendor = MACPrefix2Corp(mac);
+    if (macvendor) snprintf(vendorstr, sizeof(vendorstr), " vendor=\"%s\"", macvendor);
+    else vendorstr[0] = '\0';
+    snprintf(macascii, sizeof(macascii), "%02X:%02X:%02X:%02X:%02X:%02X",  mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
+    log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT, "MAC Address: %s (%s)\n", macascii, macvendor? macvendor : "Unknown");
+    log_write(LOG_XML, "<address addr=\"%s\"%s addrtype=\"%s\" />\n", macascii, vendorstr, "mac");
+  }
+}
 
 
 /* Prints the formatted OS Scan output to stdout, logfiles, etc (but only
@@ -1023,7 +1076,7 @@ void printosscanoutput(Target *currenths) {
 	 sprintf(p, "%hX", currenths->seq.ipids[i]);
 	 while(*p) p++;
        }
-       log_write(LOG_XML, "<ipidsequence class=\"%s\" values=\"%s\"/>\n", ipidclass2ascii(currenths->seq.ipid_seqclass), numlst);
+       log_write(LOG_XML, "<ipidsequence class=\"%s\" values=\"%s\" />\n", ipidclass2ascii(currenths->seq.ipid_seqclass), numlst);
        if (o.verbose)
 	 log_write(LOG_NORMAL|LOG_SKID|LOG_STDOUT,"IPID Sequence Generation: %s\n", ipidclass2ascii(currenths->seq.ipid_seqclass));
        log_write(LOG_MACHINE,"\tIPID Seq: %s", ipidclass2ascii(currenths->seq.ipid_seqclass));
@@ -1041,7 +1094,7 @@ void printosscanoutput(Target *currenths) {
        if (currenths->seq.ts_seqclass != TS_SEQ_UNSUPPORTED) {
 	 log_write(LOG_XML, " values=\"%s\"", numlst);
        }
-       log_write(LOG_XML, "/>\n");
+       log_write(LOG_XML, " />\n");
      }
   }
 }
